<?php

namespace Insign\PostCore\Util\OAuth2;

use Exception;
use GuzzleHttp\Client;
use Insign\PostCore\Helper\BarcodeHelper;
use Insign\PostCore\Helper\TokenHelper;
use Psr\Http\Message\ResponseInterface;

class AbstractProvider
{
    const METHOD_HTTP_POST = 'POST';
    const METHOD_HTTP_GET = 'GET';

    /** @var array  */
    protected $options = [];

    /** @var Client  */
    protected $client;

    /** @var  AccessTokenManager*/
    protected $accessTokenManager;

    /** @var  string */
    protected $oauthScope;

    /** @var TokenHelper */
    protected $tokenHelper;

    /** @var BarcodeHelper */
    protected $barcodeHelper;

    /**
     * Provider constructor.
     * @param TokenHelper $tokenHelper
     * @param BarcodeHelper $barcodeHelper
     * @param AccessTokenManager $accessTokenManager
     * @param Client $client
     * @param $oauthScope
     */
    public function __construct(
        TokenHelper $tokenHelper,
        BarcodeHelper $barcodeHelper,
        AccessTokenManager $accessTokenManager,
        Client $client,
        $oauthScope
    ) {
        $this->tokenHelper = $tokenHelper;
        $this->barcodeHelper = $barcodeHelper;
        $this->accessTokenManager = $accessTokenManager;
        $this->client = $client;
        $this->oauthScope = $oauthScope;
        if (empty($this->options)) {
            $this->options = [
                'client_id' => $this->tokenHelper->getClientId(),
                'client_secret' => $this->tokenHelper->getClientSecret(),
                'scope' => $this->oauthScope,
                'grant_type' => 'client_credentials'
            ];
        }
    }

    public function createAccessToken()
    {
        try {
            $response = $this->createRequest(
                self::METHOD_HTTP_POST,
                $this->tokenHelper->getTokenEndpoint(),
                ['form_params' => $this->options]
            );

            $data = json_decode((string)$response->getBody(), true);

            if (isset($data['access_token'])) {
                $this->accessTokenManager->setAccessToken($data['access_token'], $this->oauthScope);
                return $data['access_token'];
            }
        } catch (Exception $e) {
            throw new Exception(__('Unable to create access token "' . $e->getMessage()));
        }
    }

    public function createRequest($method, $url, array $options = []): ResponseInterface
    {
        try {
            return $this->client->request($method, $url, $options);
        } catch (Exception $e) {
            throw new Exception(__('Unable to create request "' . $e->getMessage()));
        }
    }

    public function createAuthenticatedRequest($method, $url, $token, array $options = []): ResponseInterface
    {
        $header = ['headers' => $this->getAuthorizationHeaders($token)];

        $options = array_merge_recursive($header, $options);

        try {
            return $this->client->request($method, $url, $options);
        } catch (GuzzleException $e) {
            throw new GuzzleException(__('Unable to create authenticated request "' . $e->getMessage()));
        }
    }

    protected function getAuthorizationHeaders($token = null): array
    {
        return ['Authorization' => 'Bearer ' . $token];
    }
}
